@@ -60,11 +60,15 @@ G_BEGIN_DECLS
 
 #define   REALM_DBUS_OPTION_OPERATION              "operation"
 #define   REALM_DBUS_OPTION_COMPUTER_OU            "computer-ou"
+#define   REALM_DBUS_OPTION_AUTOMATIC_ID_MAPPING   "automatic-id-mapping"
 #define   REALM_DBUS_OPTION_SERVER_SOFTWARE        "server-software"
 #define   REALM_DBUS_OPTION_CLIENT_SOFTWARE        "client-software"
 #define   REALM_DBUS_OPTION_MEMBERSHIP_SOFTWARE    "membership-software"
 #define   REALM_DBUS_OPTION_USER_PRINCIPAL         "user-principal"
 #define   REALM_DBUS_OPTION_MANAGE_SYSTEM          "manage-system"
+#define   REALM_DBUS_OPTION_COMPUTER_NAME          "computer-name"
+#define   REALM_DBUS_OPTION_OS_NAME                "os-name"
+#define   REALM_DBUS_OPTION_OS_VERSION             "os-version"
 
 #define   REALM_DBUS_IDENTIFIER_ACTIVE_DIRECTORY   "active-directory"
 #define   REALM_DBUS_IDENTIFIER_WINBIND            "winbind"
