@@ -17,6 +17,7 @@
 #include "realm-daemon.h"
 #include "realm-command.h"
 #include "realm-diagnostics.h"
+#include "realm-invocation.h"
 #include "realm-settings.h"
 
 #include <glib/gi18n-lib.h>
@@ -69,7 +70,7 @@ command_closure_free (gpointer data)
 		g_object_unref (command->invocation);
 	g_string_free (command->output, TRUE);
 	g_assert (command->source_sig == 0);
-	g_slice_free (CommandClosure, command);
+	g_free (command);
 }
 
 static void
@@ -209,7 +210,8 @@ on_process_source_input (CommandClosure *command,
 		                length - command->input_offset);
 		if (result < 0) {
 			if (errno != EINTR && errno != EAGAIN) {
-				g_warning ("couldn't write output data to process");
+				if (!command->cancelled || errno != EPIPE)
+					g_warning ("couldn't write output data to process: %s", g_strerror (errno));
 				g_bytes_unref (command->input);
 				command->input = NULL;
 				return FALSE;
@@ -407,7 +409,6 @@ realm_command_runv_async (gchar **argv,
                           gchar **environ,
                           GBytes *input,
                           GDBusMethodInvocation *invocation,
-                          GCancellable *cancellable,
                           GAsyncReadyCallback callback,
                           gpointer user_data)
 {
@@ -419,6 +420,7 @@ realm_command_runv_async (gchar **argv,
 	int error_fd = -1;
 	int input_fd = -1;
 	ProcessSource *process_source;
+	GCancellable *cancellable;
 	GSource *source;
 	gchar *cmd_string;
 	gchar *env_string;
@@ -428,9 +430,10 @@ realm_command_runv_async (gchar **argv,
 	guint i;
 
 	g_return_if_fail (argv != NULL);
-	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (invocation == NULL || G_IS_DBUS_METHOD_INVOCATION (invocation));
 
+	cancellable = realm_invocation_get_cancellable (invocation);
+
 	for (i = 0; i < NUM_FDS; i++)
 		child_fds[i] = -1;
 
@@ -471,7 +474,7 @@ realm_command_runv_async (gchar **argv,
 	g_strfreev (env);
 
 	res = g_simple_async_result_new (NULL, callback, user_data, realm_command_runv_async);
-	command = g_slice_new0 (CommandClosure);
+	command = g_new0 (CommandClosure, 1);
 	command->input = input ? g_bytes_ref (input) : NULL;
 	command->output = g_string_sized_new (128);
 	command->invocation = invocation ? g_object_ref (invocation) : NULL;
@@ -550,49 +553,53 @@ void
 realm_command_run_known_async (const gchar *known_command,
                                gchar **environ,
                                GDBusMethodInvocation *invocation,
-                               GCancellable *cancellable,
                                GAsyncReadyCallback callback,
                                gpointer user_data)
 {
 	const gchar *command_line;
+	GSimpleAsyncResult *async;
+	CommandClosure *command;
 	GError *error = NULL;
-	gchar **argv;
+	gchar *message = NULL;
+	gint exit_code = -1;
+	gchar **argv = NULL;
 	gint unused;
 
-	const gchar *empty_argv[] = {
-		"/bin/true",
-		"empty-configured-command",
-		known_command,
-		NULL,
-	};
-
-	const gchar *invalid_argv[] = {
-		"/bin/false",
-		"invalid-configured-command",
-		known_command,
-		NULL
-	};
-
 	g_return_if_fail (known_command != NULL);
-	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (invocation == NULL || G_IS_DBUS_METHOD_INVOCATION (invocation));
 
 	command_line = realm_settings_value ("commands", known_command);
 	if (command_line == NULL) {
-		g_warning ("Couldn't find the configured string commands/%s", known_command);
-		argv = g_strdupv ((gchar **)invalid_argv);
+		message = g_strdup_printf (_("Configured command not found: %s"), known_command);
+		g_warning ("Configured command not found: %s", known_command);
+		exit_code = 127;
 
 	} else if (is_only_whitespace (command_line)) {
-		argv = g_strdupv ((gchar **)empty_argv);
+		message = g_strdup_printf (_("Skipped command: %s"), known_command);
+		exit_code = 0;
 
 	} else if (!g_shell_parse_argv (command_line, &unused, &argv, &error)) {
 		g_warning ("Couldn't parse the command line: %s: %s", command_line, error->message);
 		g_error_free (error);
-		argv = g_strdupv ((gchar **)invalid_argv);
+		message = g_strdup_printf (_("Configured command invalid: %s"), command_line);
+		exit_code = 127;
+
 	}
 
-	realm_command_runv_async (argv, environ, NULL, invocation, cancellable, callback, user_data);
-	g_strfreev (argv);
+	if (message == NULL) {
+		realm_command_runv_async (argv, environ, NULL, invocation, callback, user_data);
+		g_free (argv);
+
+	} else {
+		async = g_simple_async_result_new (NULL, callback, user_data, realm_command_runv_async);
+		command = g_new0 (CommandClosure, 1);
+		command->output = g_string_new (message);
+		command->exit_code = exit_code;
+		g_simple_async_result_set_op_res_gpointer (async, command, command_closure_free);
+		g_simple_async_result_complete_in_idle (async);
+		g_object_unref (async);
+		g_free (message);
+	}
 }
 
 gint
@@ -620,3 +627,40 @@ realm_command_run_finish (GAsyncResult *result,
 
 	return command->exit_code;
 }
+
+static void
+clear_and_free_password (gpointer data)
+{
+	gchar *password = data;
+	memset ((char *)password, 0, strlen (password));
+	g_free (password);
+}
+
+GBytes *
+realm_command_build_password_line (GBytes *password)
+{
+	GByteArray *array;
+	gconstpointer data;
+	gsize length;
+	guchar *result;
+
+	array = g_byte_array_new ();
+	data = g_bytes_get_data (password, &length);
+	g_byte_array_append (array, data, length);
+
+	/*
+	 * We add a new line, which getpass() used inside command expects
+	 */
+	g_byte_array_append (array, (guchar *)"\n", 1);
+	length = array->len;
+
+	/*
+	 * In addition we add null terminator. This is not
+	 * written to 'net' command, but used by clear_and_free_password().
+	 */
+	g_byte_array_append (array, (guchar *)"\0", 1);
+
+	result = g_byte_array_free (array, FALSE);
+	return g_bytes_new_with_free_func (result, length,
+	                                   clear_and_free_password, result);
+}
