@@ -17,10 +17,13 @@
 #include "realm-settings.h"
 #include "realm-ini-config.h"
 
+#include <glib/gi18n.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
 
 #include <string.h>
+#include <errno.h>
 
 #define REALM_INI_CONFIG_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST ((klass), REALM_TYPE_INI_CONFIG, RealmIniConfigClass))
 #define REALM_IS_INI_CONFIG_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE ((klass), REALM_TYPE_INI_CONFIG))
@@ -99,7 +102,7 @@ config_section_free (gpointer data)
 {
 	ConfigSection *sect = data;
 	g_hash_table_destroy (sect->parameters);
-	g_slice_free (ConfigSection, sect);
+	g_free (sect);
 }
 
 static void
@@ -108,7 +111,7 @@ config_line_free (gpointer data)
 	ConfigLine *line = data;
 	g_free (line->name);
 	g_bytes_unref (line->bytes);
-	g_slice_free (ConfigLine, line);
+	g_free (line);
 }
 
 static void
@@ -462,7 +465,7 @@ parse_config_line (RealmIniConfig *self,
 	gchar *name = NULL;
 	gint type;
 
-	line = g_slice_new0 (ConfigLine);
+	line = g_new0 (ConfigLine, 1);
 	line->bytes = g_bytes_ref (bytes);
 
 	/* What kind of line is this? */
@@ -471,7 +474,7 @@ parse_config_line (RealmIniConfig *self,
 	case SECTION:
 		sect = g_hash_table_lookup (self->sections, name);
 		if (sect == NULL) {
-			sect = g_slice_new0 (ConfigSection);
+			sect = g_new0 (ConfigSection, 1);
 			sect->parameters = g_hash_table_new (conf_str_hash, conf_str_equal);
 			g_hash_table_replace (self->sections, name, sect);
 			sect->head = line;
@@ -704,6 +707,52 @@ realm_ini_config_write_file (RealmIniConfig *self,
 	return ret;
 }
 
+gboolean
+realm_ini_config_write_fd (RealmIniConfig *self,
+                           gint fd,
+                           GError **error)
+{
+	GBytes *bytes;
+	gboolean ret = TRUE;
+	const gchar *contents;
+	gint result;
+	gsize length;
+	gint errn;
+
+	g_return_val_if_fail (REALM_IS_INI_CONFIG (self), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	bytes = realm_ini_config_write_bytes (self);
+	g_return_val_if_fail (bytes != NULL, FALSE);
+
+	contents = g_bytes_get_data (bytes, &length);
+
+	while (length > 0) {
+		result = write (fd, contents, length);
+		if (result < 0) {
+			if (errno != EINTR && errno != EAGAIN) {
+				errn = errno;
+				g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errn),
+				             _("Couldn't write out config: %s"), g_strerror (errn));
+				ret = FALSE;
+				break;
+			}
+
+			result = 0;
+		}
+
+		g_return_val_if_fail (result <= length, FALSE);
+		contents += result;
+		length -= result;
+	}
+
+	g_bytes_unref (bytes);
+
+	if (ret)
+		realm_ini_config_set_filename (self, NULL);
+	return ret;
+}
+
 static void
 config_set_value (RealmIniConfig *self,
                   const gchar *section,
@@ -728,20 +777,20 @@ config_set_value (RealmIniConfig *self,
 			return;
 
 		/* A blank line */
-		line = g_slice_new0 (ConfigLine);
+		line = g_new0 (ConfigLine, 1);
 		line->bytes = g_bytes_new ("\n", 1);
 		line->name = NULL;
 		append_config_line (self, line);
 
 		/* The actual section header */
 		data = g_strdup_printf ("[%s]\n", section);
-		line = g_slice_new0 (ConfigLine);
+		line = g_new0 (ConfigLine, 1);
 		line->bytes = g_bytes_new_take (data, strlen (data));
 		line->name = g_strdup (section);
 		append_config_line (self, line);
 
 		/* Register it */
-		sect = g_slice_new0 (ConfigSection);
+		sect = g_new0 (ConfigSection, 1);
 		sect->parameters = g_hash_table_new (conf_str_hash, conf_str_equal);
 		sect->head = sect->tail = line;
 		g_hash_table_replace (self->sections, line->name, sect);
@@ -770,7 +819,7 @@ config_set_value (RealmIniConfig *self,
 
 	/* Don't have this line, add to section */
 	if (line == NULL) {
-		line = g_slice_new0 (ConfigLine);
+		line = g_new0 (ConfigLine, 1);
 		line->bytes = g_bytes_new_take (data, strlen (data));
 		line->name = g_strdup (name);
 		insert_config_line (self, sect->tail, line);
@@ -787,12 +836,23 @@ void
 realm_ini_config_set (RealmIniConfig *self,
                       const gchar *section,
                       const gchar *name,
-                      const gchar *value)
+                      const gchar *value,
+                      ...)
 {
+	va_list va;
+
 	g_return_if_fail (REALM_IS_INI_CONFIG (self));
 	g_return_if_fail (section != NULL);
 
-	config_set_value (self, section, name, value);
+	va_start (va, value);
+	while (name != NULL) {
+		config_set_value (self, section, name, value);
+		name = va_arg (va, const gchar *);
+		if (name != NULL)
+			value = va_arg (va, const gchar *);
+	}
+	va_end (va);
+
 	if (!self->changing)
 		g_signal_emit (self, signals[CHANGED], 0);
 }
@@ -820,6 +880,29 @@ realm_ini_config_get (RealmIniConfig *self,
 	return parse_config_line_value (self, line->bytes);
 }
 
+gboolean
+realm_ini_config_have (RealmIniConfig *self,
+                       const gchar *section,
+                       const gchar *name)
+{
+	ConfigSection *sect;
+	ConfigLine *line;
+
+	g_return_val_if_fail (REALM_IS_INI_CONFIG (self), FALSE);
+	g_return_val_if_fail (section != NULL, FALSE);
+	g_return_val_if_fail (name != NULL, FALSE);
+
+	sect = g_hash_table_lookup (self->sections, section);
+	if (sect == NULL)
+		return FALSE;
+
+	line = g_hash_table_lookup (sect->parameters, name);
+	if (line == NULL)
+		return FALSE;
+
+	return TRUE;
+}
+
 GHashTable *
 realm_ini_config_get_all (RealmIniConfig *self,
                           const gchar *section)
@@ -909,7 +992,7 @@ realm_ini_config_set_list (RealmIniConfig *self,
 	g_return_if_fail (delimiter != NULL);
 
 	value = g_strjoinv (delimiter, (gchar **)values);
-	realm_ini_config_set (self, section, name, value);
+	realm_ini_config_set (self, section, name, value, NULL);
 	g_free (value);
 }
 
@@ -967,6 +1050,64 @@ realm_ini_config_set_list_diff (RealmIniConfig *self,
 	g_free (delim);
 }
 
+gchar **
+realm_ini_config_get_sections (RealmIniConfig *self)
+{
+	GHashTableIter iter;
+	gpointer section;
+	gchar **sections;
+	gint i = 0;
+
+	g_return_val_if_fail (REALM_IS_INI_CONFIG (self), NULL);
+
+	sections = g_new0 (gchar *, g_hash_table_size (self->sections) + 1);
+	g_hash_table_iter_init (&iter, self->sections);
+	while (g_hash_table_iter_next (&iter, &section, NULL))
+		sections[i++] = g_strdup (section);
+
+	return sections;
+}
+
+gboolean
+realm_ini_config_get_boolean (RealmIniConfig *config,
+                              const gchar *section,
+                              const gchar *name,
+                              gboolean defahlt)
+{
+	gboolean ret;
+	gchar *value;
+
+	g_return_val_if_fail (REALM_IS_INI_CONFIG (config), FALSE);
+	g_return_val_if_fail (section != NULL, FALSE);
+	g_return_val_if_fail (name != NULL, FALSE);
+
+	value = realm_ini_config_get (config, section, name);
+	if (value == NULL) {
+		ret = defahlt;
+	} else if (g_ascii_strcasecmp (value, "true") == 0) {
+		ret = TRUE;
+	} else if (g_ascii_strcasecmp (value, "false") == 0) {
+		ret = FALSE;
+	} else if (config->flags & REALM_INI_STRICT_BOOLEAN) {
+		g_warning ("Invalid %s boolean value for %s field: %s",
+		           value, config->filename ? config->filename : "", name);
+		ret = defahlt;
+	} else if (g_ascii_strcasecmp (value, "1") == 0 ||
+	           g_ascii_strcasecmp (value, "yes") == 0) {
+		ret = TRUE;
+	} else if (g_ascii_strcasecmp (value, "0") == 0 ||
+	           g_ascii_strcasecmp (value, "no") == 0) {
+		ret = FALSE;
+	} else {
+		g_warning ("Invalid %s boolean value for %s field: %s",
+		           value, config->filename ? config->filename : "", name);
+		ret = defahlt;
+	}
+
+	g_free (value);
+	return ret;
+}
+
 gboolean
 realm_ini_config_have_section (RealmIniConfig *self,
                                const gchar *section)
@@ -1017,8 +1158,6 @@ realm_ini_config_remove_section (RealmIniConfig *self,
 			head = head->prev;
 	}
 
-	g_hash_table_remove (self->sections, section);
-
 	/* Remove this section from the config file */
 	if (head->prev)
 		head->prev->next = tail->next;
@@ -1031,10 +1170,15 @@ realm_ini_config_remove_section (RealmIniConfig *self,
 	head->prev = NULL;
 	tail->next = NULL;
 
+	g_hash_table_remove (self->sections, section);
+
 	for (line = head; line != NULL; line = next) {
 		next = line->next;
 		config_line_free (line);
 	}
+
+	if (!self->changing)
+		g_signal_emit (self, signals[CHANGED], 0);
 }
 
 gboolean
