@@ -479,8 +479,6 @@ on_install_resolved (GObject *source,
 			packages_install_async (install->connection,
 			                        (const gchar **)package_ids, cancellable,
 			                        on_install_installed, g_object_ref (task));
-			if (cancellable)
-				g_object_unref (cancellable);
 		}
 
 		g_free (missing);
@@ -615,6 +613,7 @@ realm_packages_install_async (const gchar **package_sets,
 	task = g_task_new (NULL, NULL, callback, user_data);
 	install = g_new0 (InstallClosure, 1);
 	install->automatic = realm_options_automatic_install ();
+	install->invocation = invocation ? g_object_ref (invocation) : NULL;
 	install->connection = g_object_ref (connection);
 	g_task_set_task_data (task, install, install_closure_free);
 
@@ -648,7 +647,6 @@ realm_packages_install_async (const gchar **package_sets,
 		cancellable = realm_invocation_get_cancellable (install->invocation);
 		packages_resolve_async (connection, (const gchar **)install->packages, cancellable,
 		                        on_install_resolved, g_object_ref (task));
-		g_object_unref (cancellable);
 	}
 
 	g_object_unref (task);
