@@ -183,6 +183,13 @@ on_join_do_winbind (GObject *source,
 	GHashTable *settings = NULL;
 	GError *error = NULL;
 	const gchar *name;
+	const gchar *computer_name;
+
+	computer_name = realm_options_computer_name (enroll->options, enroll->disco->domain_name);
+	/* Use truncated name if set and explicit name is not available */
+	if (enroll->disco->explicit_netbios && computer_name == NULL)
+		computer_name = enroll->disco->explicit_netbios;
+
 
 	realm_samba_enroll_join_finish (result, &error);
 	if (error == NULL) {
@@ -192,15 +199,16 @@ on_join_do_winbind (GObject *source,
 		                         "workgroup", enroll->disco->workgroup,
 		                         "template homedir", realm_settings_string ("users", "default-home"),
 		                         "template shell", realm_settings_string ("users", "default-shell"),
-		                         "netbios name", enroll->disco->explicit_netbios,
+		                         "netbios name", computer_name,
 		                         "password server", enroll->disco->explicit_server,
 		                         "kerberos method", "system keytab",
 		                         NULL);
 	}
 
+
 	if (error == NULL) {
 		name = realm_kerberos_get_name (REALM_KERBEROS (self));
-		realm_samba_winbind_configure_async (self->config, name,
+		realm_samba_winbind_configure_async (self->config, name, enroll->options,
 		                                     enroll->invocation,
 		                                     on_winbind_done, g_object_ref (task));
 	} else {
@@ -285,7 +293,8 @@ realm_samba_join_async (RealmKerberosMembership *membership,
 		g_task_return_error (task, error);
 
 	} else {
-		realm_packages_install_async (SAMBA_PACKAGES, enroll->invocation, options,
+		realm_packages_install_async (SAMBA_PACKAGES, enroll->invocation,
+		                              g_dbus_method_invocation_get_connection (enroll->invocation),
 		                              on_install_do_join, g_object_ref (task));
 	}
 
@@ -293,6 +302,19 @@ realm_samba_join_async (RealmKerberosMembership *membership,
 	g_object_unref (task);
 }
 
+static const RealmCredential *
+realm_samba_join_creds (RealmKerberosMembership *self)
+{
+	static const RealmCredential creds[] = {
+		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_ADMIN },
+		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_USER },
+		{ REALM_CREDENTIAL_CCACHE, REALM_CREDENTIAL_OWNER_ADMIN },
+		{ 0, },
+	};
+
+	return creds;
+}
+
 typedef struct {
 	GDBusMethodInvocation *invocation;
 	RealmDisco *disco;
@@ -425,6 +447,19 @@ realm_samba_leave_async (RealmKerberosMembership *membership,
 	g_object_unref (task);
 }
 
+static const RealmCredential *
+realm_samba_leave_creds (RealmKerberosMembership *self)
+{
+	static const RealmCredential creds[] = {
+		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_ADMIN },
+		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_USER },
+		{ REALM_CREDENTIAL_AUTOMATIC, REALM_CREDENTIAL_OWNER_NONE },
+		{ 0, },
+	};
+
+	return creds;
+}
+
 static gboolean
 realm_samba_change_logins (RealmKerberos *realm,
                            GDBusMethodInvocation *invocation,
@@ -674,33 +709,13 @@ realm_samba_class_init (RealmSambaClass *klass)
 static void
 realm_samba_kerberos_membership_iface (RealmKerberosMembershipIface *iface)
 {
-	/*
-	 * Each line is a combination of owner and what kind of credentials are supported,
-	 * same for enroll/leave. We can't accept a ccache, because samba3 needs
-	 * to have credentials limited to RC4.
-	 */
-
-	static const RealmCredential join_supported[] = {
-		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_ADMIN },
-		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_USER },
-		{ REALM_CREDENTIAL_CCACHE, REALM_CREDENTIAL_OWNER_ADMIN },
-		{ 0, },
-	};
-
-	static const RealmCredential leave_supported[] = {
-		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_ADMIN },
-		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_USER },
-		{ REALM_CREDENTIAL_AUTOMATIC, REALM_CREDENTIAL_OWNER_NONE },
-		{ 0, },
-	};
-
 	iface->join_async = realm_samba_join_async;
 	iface->join_finish = realm_samba_membership_generic_finish;
-	iface->join_creds_supported = join_supported;
+	iface->join_creds = realm_samba_join_creds;
 
 	iface->leave_async = realm_samba_leave_async;
 	iface->leave_finish = realm_samba_membership_generic_finish;
-	iface->leave_creds_supported = leave_supported;
+	iface->leave_creds = realm_samba_leave_creds;
 }
 
 RealmKerberos *
