@@ -163,6 +163,7 @@ configure_sssd_for_domain (RealmIniConfig *config,
 	GString *realmd_tags;
 	const gchar *access_provider;
 	const gchar *shell;
+    const gchar *explicit_computer_name;
 	gchar *authid = NULL;
 	gboolean qualify;
 	gboolean ret;
@@ -172,17 +173,19 @@ configure_sssd_for_domain (RealmIniConfig *config,
 	home = realm_sssd_build_default_home (realm_settings_string ("users", "default-home"));
 	qualify = realm_options_qualify_names (disco->domain_name);
 	shell = realm_settings_string ("users", "default-shell");
-
+	explicit_computer_name = realm_options_computer_name (options, disco->domain_name);
 	realmd_tags = g_string_new ("");
 	if (realm_options_manage_system (options, disco->domain_name))
 		g_string_append (realmd_tags, "manages-system ");
 	g_string_append (realmd_tags, use_adcli ? "joined-with-adcli " : "joined-with-samba ");
 
 	/*
-	 * Explicitly set the netbios authid for sssd to use in this case, since
+	 * Explicitly set the netbios authid for sssd to use in these cases, since
 	 * otherwise sssd won't know which kerberos principal to use
 	 */
-	if (disco->explicit_netbios)
+	if (explicit_computer_name != NULL)
+		authid = g_strdup_printf ("%s$", explicit_computer_name);
+	else if (disco->explicit_netbios)
 		authid = g_strdup_printf ("%s$", disco->explicit_netbios);
 
 	ret = realm_sssd_config_add_domain (config, disco->domain_name, error,
@@ -194,7 +197,7 @@ configure_sssd_for_domain (RealmIniConfig *config,
 	                                    "ad_domain", disco->domain_name,
 	                                    "krb5_realm", disco->kerberos_realm,
 	                                    "krb5_store_password_if_offline", "True",
-	                                    "ldap_id_mapping", realm_options_automatic_mapping (disco->domain_name) ? "True" : "False",
+	                                    "ldap_id_mapping", realm_options_automatic_mapping (options, disco->domain_name) ? "True" : "False",
 	                                    "realmd_tags", realmd_tags->str,
 
 	                                    "fallback_homedir", home,
@@ -343,26 +346,13 @@ parse_join_options (JoinClosure *join,
 		}
 
 	/*
-	 * If we are enrolling with a ccache, then prefer to use adcli over samba.
-	 * There have been some strange corner case problems when using samba with
-	 * a ccache.
+	 * For other valid types of credentials we prefer adcli.
 	 */
-	} else if (cred->type == REALM_CREDENTIAL_CCACHE) {
+	} else if (cred->type == REALM_CREDENTIAL_CCACHE ||
+	           (cred->type == REALM_CREDENTIAL_PASSWORD && cred->owner == REALM_CREDENTIAL_OWNER_ADMIN)) {
 		if (!software)
 			software = REALM_DBUS_IDENTIFIER_ADCLI;
 
-	/*
-	 * For other supported enrolling credentials, we support either adcli or
-	 * samba. But since adcli is pretty immature at this point, we use samba
-	 * by default. Samba falls over with hostnames that are not perfectly
-	 * specified, so use adcli there.
-	 */
-	} else if (cred->type == REALM_CREDENTIAL_PASSWORD && cred->owner == REALM_CREDENTIAL_OWNER_ADMIN) {
-		if (!software && join->disco->explicit_server)
-			software = REALM_DBUS_IDENTIFIER_ADCLI;
-		else if (!software)
-			software = REALM_DBUS_IDENTIFIER_SAMBA;
-
 	/* It would be odd to get here */
 	} else {
 		g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS,
@@ -419,13 +409,47 @@ realm_sssd_ad_join_async (RealmKerberosMembership *membership,
 
 	/* Prepared successfully without an error */
 	} else {
-		realm_packages_install_async (join->packages, join->invocation, options,
+		realm_packages_install_async (join->packages, join->invocation,
+		                              g_dbus_method_invocation_get_connection (join->invocation),
 		                              on_install_do_join, g_object_ref (task));
 	}
 
 	g_object_unref (task);
 }
 
+static const RealmCredential *
+realm_sssd_ad_join_creds (RealmKerberosMembership *membership)
+{
+	/*
+	 * Each line is a combination of owner and what kind of credentials are supported,
+	 * same for enroll/leave. We can't accept a ccache with samba because of certain
+	 * corner cases. However we do accept ccache for an admin user, and then we use
+	 * adcli with that ccache.
+	 */
+
+	static const RealmCredential creds[] = {
+		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_ADMIN, },
+		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_USER, },
+		{ REALM_CREDENTIAL_CCACHE, REALM_CREDENTIAL_OWNER_ADMIN, },
+		{ REALM_CREDENTIAL_AUTOMATIC, REALM_CREDENTIAL_OWNER_NONE, },
+		{ REALM_CREDENTIAL_SECRET, REALM_CREDENTIAL_OWNER_NONE, },
+		{ 0, },
+	};
+
+	static const RealmCredential creds_no_auto[] = {
+		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_ADMIN, },
+		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_USER, },
+		{ REALM_CREDENTIAL_CCACHE, REALM_CREDENTIAL_OWNER_ADMIN, },
+		{ REALM_CREDENTIAL_SECRET, REALM_CREDENTIAL_OWNER_NONE, },
+		{ 0, }
+	};
+
+	const gchar *name;
+
+	name = realm_kerberos_get_name (REALM_KERBEROS (membership));
+	return realm_options_automatic_join (name) ? creds : creds_no_auto;
+}
+
 typedef struct {
 	GDBusMethodInvocation *invocation;
 	gchar *realm_name;
@@ -527,6 +551,20 @@ realm_sssd_ad_leave_async (RealmKerberosMembership *membership,
 	g_object_unref (task);
 }
 
+static const RealmCredential *
+realm_sssd_ad_leave_creds (RealmKerberosMembership *membership)
+{
+	/* For leave, we don't support one-time-password (ie: secret/none) */
+	static const RealmCredential creds[] = {
+		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_ADMIN, },
+		{ REALM_CREDENTIAL_CCACHE, REALM_CREDENTIAL_OWNER_ADMIN, },
+		{ REALM_CREDENTIAL_AUTOMATIC, REALM_CREDENTIAL_OWNER_NONE, },
+		{ 0, },
+	};
+
+	return creds;
+}
+
 static gboolean
 realm_sssd_ad_generic_finish (RealmKerberosMembership *realm,
                               GAsyncResult *result,
@@ -594,35 +632,11 @@ realm_sssd_ad_class_init (RealmSssdAdClass *klass)
 static void
 realm_sssd_ad_kerberos_membership_iface (RealmKerberosMembershipIface *iface)
 {
-	/*
-	 * Each line is a combination of owner and what kind of credentials are supported,
-	 * same for enroll/leave. We can't accept a ccache with samba because of certain
-	 * corner cases. However we do accept ccache for an admin user, and then we use
-	 * adcli with that ccache.
-	 */
-
-	static const RealmCredential join_supported[] = {
-		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_ADMIN, },
-		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_USER, },
-		{ REALM_CREDENTIAL_CCACHE, REALM_CREDENTIAL_OWNER_ADMIN, },
-		{ REALM_CREDENTIAL_AUTOMATIC, REALM_CREDENTIAL_OWNER_NONE, },
-		{ REALM_CREDENTIAL_SECRET, REALM_CREDENTIAL_OWNER_NONE, },
-		{ 0, },
-	};
-
-	/* For leave, we don't support one-time-password (ie: secret/none) */
-	static const RealmCredential leave_supported[] = {
-		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_ADMIN, },
-		{ REALM_CREDENTIAL_CCACHE, REALM_CREDENTIAL_OWNER_ADMIN, },
-		{ REALM_CREDENTIAL_AUTOMATIC, REALM_CREDENTIAL_OWNER_NONE, },
-		{ 0, },
-	};
-
 	iface->join_async = realm_sssd_ad_join_async;
 	iface->join_finish = realm_sssd_ad_generic_finish;
-	iface->join_creds_supported = join_supported;
+	iface->join_creds = realm_sssd_ad_join_creds;
 
 	iface->leave_async = realm_sssd_ad_leave_async;
 	iface->leave_finish = realm_sssd_ad_generic_finish;
-	iface->leave_creds_supported = leave_supported;
+	iface->leave_creds = realm_sssd_ad_leave_creds;
 }
